# azure-waf-deployer/azure_waf_deployer/templates.py
"""
Template management for Azure WAF deployments
"""

import json
from typing import Dict, Any
from pathlib import Path
import pkg_resources

from .exceptions import TemplateError


class TemplateManager:
    """Manages ARM and Bicep templates for WAF deployments"""
    
    def __init__(self):
        self.template_path = Path(__file__).parent / "templates"
    
    def get_template(self, template_name: str) -> Dict[str, Any]:
        """
        Get ARM template by name
        
        Args:
            template_name: Name of the template
            
        Returns:
            ARM template as dictionary
        """
        try:
            template_file = f"{template_name}.json"
            template_content = pkg_resources.resource_string(
                'azure_waf_deployer', 
                f'templates/{template_file}'
            ).decode('utf-8')
            return json.loads(template_content)
        except Exception as e:
            raise TemplateError(f"Failed to load template {template_name}: {str(e)}")
    
    def list_templates(self) -> list:
        """List available templates"""
        try:
            templates_dir = Path(pkg_resources.resource_filename('azure_waf_deployer', 'templates'))
            return [f.stem for f in templates_dir.glob("*.json")]
        except Exception as e:
            raise TemplateError(f"Failed to list templates: {str(e)}")


# azure-waf-deployer/azure_waf_deployer/templates/application_gateway.json
{
    "$schema": "https://schema.management.azure.com/schemas/2019-04-01/deploymentTemplate.json#",
    "contentVersion": "1.0.0.0",
    "parameters": {
        "applicationGatewayName": {
            "type": "string",
            "metadata": {
                "description": "Name of the Application Gateway"
            }
        },
        "location": {
            "type": "string",
            "defaultValue": "[resourceGroup().location]",
            "metadata": {
                "description": "Location for all resources"
            }
        },
        "skuTier": {
            "type": "string",
            "defaultValue": "WAF_v2",
            "allowedValues": [
                "WAF_v2",
                "Standard_v2"
            ],
            "metadata": {
                "description": "SKU tier for the Application Gateway"
            }
        },
        "skuName": {
            "type": "string",
            "defaultValue": "WAF_v2",
            "allowedValues": [
                "WAF_v2",
                "Standard_v2"
            ],
            "metadata": {
                "description": "SKU name for the Application Gateway"
            }
        },
        "capacity": {
            "type": "int",
            "defaultValue": 2,
            "minValue": 1,
            "maxValue": 32,
            "metadata": {
                "description": "Capacity (instance count) of Application Gateway"
            }
        },
        "wafMode": {
            "type": "string",
            "defaultValue": "Prevention",
            "allowedValues": [
                "Detection",
                "Prevention"
            ],
            "metadata": {
                "description": "WAF mode"
            }
        },
        "vnetName": {
            "type": "string",
            "defaultValue": "[concat(parameters('applicationGatewayName'), '-vnet')]",
            "metadata": {
                "description": "Virtual network name"
            }
        },
        "subnetName": {
            "type": "string",
            "defaultValue": "appgw-subnet",
            "metadata": {
                "description": "Subnet name for Application Gateway"
            }
        },
        "publicIpName": {
            "type": "string",
            "defaultValue": "[concat(parameters('applicationGatewayName'), '-pip')]",
            "metadata": {
                "description": "Public IP name"
            }
        },
        "backendPools": {
            "type": "array",
            "defaultValue": [],
            "metadata": {
                "description": "Backend pools configuration"
            }
        },
        "managedRuleSets": {
            "type": "array",
            "defaultValue": [
                {
                    "ruleSetType": "Microsoft_DefaultRuleSet",
                    "ruleSetVersion": "2.1"
                },
                {
                    "ruleSetType": "Microsoft_BotManagerRuleSet",
                    "ruleSetVersion": "1.0"
                }
            ],
            "metadata": {
                "description": "Managed rule sets"
            }
        },
        "customRules": {
            "type": "array",
            "defaultValue": [],
            "metadata": {
                "description": "Custom WAF rules"
            }
        },
        "enableHttp2": {
            "type": "bool",
            "defaultValue": true,
            "metadata": {
                "description": "Enable HTTP2 support"
            }
        },
        "requestTimeout": {
            "type": "int",
            "defaultValue": 20,
            "metadata": {
                "description": "Request timeout in seconds"
            }
        }
    },
    "variables": {
        "wafPolicyName": "[concat(parameters('applicationGatewayName'), '-wafpolicy')]",
        "vnetAddressPrefix": "10.0.0.0/16",
        "subnetAddressPrefix": "10.0.1.0/24",
        "publicIPRef": "[resourceId('Microsoft.Network/publicIPAddresses', parameters('publicIpName'))]",
        "subnetRef": "[resourceId('Microsoft.Network/virtualNetworks/subnets', parameters('vnetName'), parameters('subnetName'))]",
        "wafPolicyRef": "[resourceId('Microsoft.Network/ApplicationGatewayWebApplicationFirewallPolicies', variables('wafPolicyName'))]"
    },
    "resources": [
        {
            "type": "Microsoft.Network/virtualNetworks",
            "apiVersion": "2022-07-01",
            "name": "[parameters('vnetName')]",
            "location": "[parameters('location')]",
            "properties": {
                "addressSpace": {
                    "addressPrefixes": [
                        "[variables('vnetAddressPrefix')]"
                    ]
                },
                "subnets": [
                    {
                        "name": "[parameters('subnetName')]",
                        "properties": {
                            "addressPrefix": "[variables('subnetAddressPrefix')]"
                        }
                    }
                ]
            }
        },
        {
            "type": "Microsoft.Network/publicIPAddresses",
            "apiVersion": "2022-07-01",
            "name": "[parameters('publicIpName')]",
            "location": "[parameters('location')]",
            "sku": {
                "name": "Standard",
                "tier": "Regional"
            },
            "properties": {
                "publicIPAllocationMethod": "Static"
            }
        },
        {
            "type": "Microsoft.Network/ApplicationGatewayWebApplicationFirewallPolicies",
            "apiVersion": "2022-07-01",
            "name": "[variables('wafPolicyName')]",
            "location": "[parameters('location')]",
            "properties": {
                "policySettings": {
                    "mode": "[parameters('wafMode')]",
                    "state": "Enabled",
                    "maxRequestBodySizeInKb": 128,
                    "fileUploadLimitInMb": 100,
                    "requestBodyCheck": true
                },
                "managedRules": {
                    "managedRuleSets": "[parameters('managedRuleSets')]"
                },
                "customRules": "[parameters('customRules')]"
            }
        },
        {
            "type": "Microsoft.Network/applicationGateways",
            "apiVersion": "2022-07-01",
            "name": "[parameters('applicationGatewayName')]",
            "location": "[parameters('location')]",
            "dependsOn": [
                "[variables('wafPolicyRef')]",
                "[variables('publicIPRef')]",
                "[variables('subnetRef')]"
            ],
            "properties": {
                "sku": {
                    "name": "[parameters('skuName')]",
                    "tier": "[parameters('skuTier')]"
                },
                "autoscaleConfiguration": {
                    "minCapacity": 1,
                    "maxCapacity": 32
                },
                "gatewayIPConfigurations": [
                    {
                        "name": "appGatewayIpConfig",
                        "properties": {
                            "subnet": {
                                "id": "[variables('subnetRef')]"
                            }
                        }
                    }
                ],
                "frontendIPConfigurations": [
                    {
                        "name": "appGatewayFrontendIP",
                        "properties": {
                            "publicIPAddress": {
                                "id": "[variables('publicIPRef')]"
                            }
                        }
                    }
                ],
                "frontendPorts": [
                    {
                        "name": "port_80",
                        "properties": {
                            "port": 80
                        }
                    },
                    {
                        "name": "port_443",
                        "properties": {
                            "port": 443
                        }
                    }
                ],
                "backendAddressPools": "[if(empty(parameters('backendPools')), createArray(createObject('name', 'defaultBackendPool', 'properties', createObject())), parameters('backendPools'))]",
                "backendHttpSettingsCollection": [
                    {
                        "name": "defaultHttpSettings",
                        "properties": {
                            "port": 80,
                            "protocol": "Http",
                            "cookieBasedAffinity": "Disabled",
                            "requestTimeout": "[parameters('requestTimeout')]"
                        }
                    }
                ],
                "httpListeners": [
                    {
                        "name": "defaultListener",
                        "properties": {
                            "frontendIPConfiguration": {
                                "id": "[resourceId('Microsoft.Network/applicationGateways/frontendIPConfigurations', parameters('applicationGatewayName'), 'appGatewayFrontendIP')]"
                            },
                            "frontendPort": {
                                "id": "[resourceId('Microsoft.Network/applicationGateways/frontendPorts', parameters('applicationGatewayName'), 'port_80')]"
                            },
                            "protocol": "Http"
                        }
                    }
                ],
                "requestRoutingRules": [
                    {
                        "name": "defaultRule",
                        "properties": {
                            "ruleType": "Basic",
                            "priority": 100,
                            "httpListener": {
                                "id": "[resourceId('Microsoft.Network/applicationGateways/httpListeners', parameters('applicationGatewayName'), 'defaultListener')]"
                            },
                            "backendAddressPool": {
                                "id": "[resourceId('Microsoft.Network/applicationGateways/backendAddressPools', parameters('applicationGatewayName'), if(empty(parameters('backendPools')), 'defaultBackendPool', parameters('backendPools')[0].name))]"
                            },
                            "backendHttpSettings": {
                                "id": "[resourceId('Microsoft.Network/applicationGateways/backendHttpSettingsCollection', parameters('applicationGatewayName'), 'defaultHttpSettings')]"
                            }
                        }
                    }
                ],
                "enableHttp2": "[parameters('enableHttp2')]",
                "firewallPolicy": {
                    "id": "[variables('wafPolicyRef')]"
                }
            }
        }
    ],
    "outputs": {
        "applicationGatewayId": {
            "type": "string",
            "value": "[resourceId('Microsoft.Network/applicationGateways', parameters('applicationGatewayName'))]"
        },
        "publicIPAddress": {
            "type": "string",
            "value": "[reference(variables('publicIPRef')).ipAddress]"
        },
        "wafPolicyId": {
            "type": "string",
            "value": "[variables('wafPolicyRef')]"
        },
        "fqdn": {
            "type": "string",
            "value": "[reference(variables('publicIPRef')).dnsSettings.fqdn]"
        }
    }
}


# azure-waf-deployer/azure_waf_deployer/cli.py
"""
Command-line interface for Azure WAF deployer
"""

import click
import logging
import json
from pathlib import Path
from rich.console import Console
from rich.table import Table
from rich.progress import Progress, SpinnerColumn, TextColumn

from .deployer import WAFDeployer
from .config import WAFConfig
from .exceptions import WAFDeploymentError, ConfigurationError

console = Console()
logger = logging.getLogger(__name__)


@click.group()
@click.option('--verbose', '-v', is_flag=True, help='Enable verbose logging')
def main(verbose):
    """Azure WAF Deployer - Deploy WAF-enabled infrastructure on Azure"""
    level = logging.DEBUG if verbose else logging.INFO
    logging.basicConfig(level=level, format='%(asctime)s - %(levelname)s - %(message)s')


@main.command()
@click.option('--subscription-id', required=True, help='Azure subscription ID')
@click.option('--resource-group', required=True, help='Resource group name')
@click.option('--config', '-c', required=True, help='Configuration file path')
@click.option('--template-type', default='application_gateway', 
              type=click.Choice(['application_gateway', 'front_door']),
              help='Template type to deploy')
@click.option('--validate-only', is_flag=True, help='Only validate template, don\'t deploy')
def deploy(subscription_id, resource_group, config, template_type, validate_only):
    """Deploy WAF-enabled infrastructure"""
    try:
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            console=console
        ) as progress:
            task = progress.add_task("Loading configuration...", total=None)
            
            # Load configuration
            config = WAFConfig.from_yaml(config)
            progress.update(task, description="Initializing deployer...")
            
            # Initialize deployer
            deployer = WAFDeployer(subscription_id)
            
            if validate_only:
                progress.update(task, description="Validating template...")
                is_valid = deployer.validate_template(resource_group, config, template_type)
                if is_valid:
                    console.print("✅ Template validation successful!", style="green")
                else:
                    console.print("❌ Template validation failed!", style="red")
                    return
            else:
                progress.update(task, description=f"Deploying {template_type}...")
                
                if template_type == 'application_gateway':
                    result = deployer.deploy_application_gateway_waf(resource_group, config, template_type)
                elif template_type == 'front_door':
                    result = deployer.deploy_front_door_waf(resource_group, config, template_type)
                
                progress.update(task, description="Deployment complete!")
                
                # Display results
                console.print(f"✅ Deployment successful!", style="green")
                console.print(f"Deployment name: {result['deployment_name']}")
                console.print(f"Resource group: {result['resource_group']}")
                
                if result.get('outputs'):
                    console.print("\nOutputs:")
                    for key, value in result['outputs'].items():
                        console.print(f"  {key}: {value['value']}")
                        
    except Exception as e:
        console.print(f"❌ Error: {str(e)}", style="red")
        raise click.Abort()


@main.command()
@click.option('--name', required=True, help='WAF configuration name')
@click.option('--location', required=True, help='Azure region')
@click.option('--output', '-o', required=True, help='Output configuration file')
def init(name, location, output):
    """Initialize a new WAF configuration file"""
    try:
        config = WAFConfig(name=name, location=location)
        config.to_yaml(output)
        console.print(f"✅ Configuration file created: {output}", style="green")
    except Exception as e:
        console.print(f"❌ Error creating configuration: {str(e)}", style="red")
        raise click.Abort()


@main.command()
@click.option('--subscription-id', required=True, help='Azure subscription ID')
@click.option('--resource-group', required=True, help='Resource group name')
@click.option('--deployment-name', required=True, help='Deployment name')
def status(subscription_id, resource_group, deployment_name):
    """Check deployment status"""
    try:
        deployer = WAFDeployer(subscription_id)
        result = deployer.get_deployment_status(resource_group, deployment_name)
        
        if 'error' in result:
            console.print(f"❌ Error: {result['error']}", style="red")
        else:
            table = Table(title="Deployment Status")
            table.add_column("Property", style="cyan")
            table.add_column("Value", style="magenta")
            
            table.add_row("Name", result['name'])
            table.add_row("Status", result['provisioning_state'])
            table.add_row("Timestamp", str(result['timestamp']))
            
            console.print(table)
            
    except Exception as e:
        console.print(f"❌ Error: {str(e)}", style="red")
        raise click.Abort()


@main.command()
def list_templates():
    """List available templates"""
    try:
        from .templates import TemplateManager
        manager = TemplateManager()
        templates = manager.list_templates()
        
        table = Table(title="Available Templates")
        table.add_column("Template Name", style="cyan")
        table.add_column("Description", style="magenta")
        
        descriptions = {
            'application_gateway': 'Application Gateway with WAF v2',
            'front_door': 'Azure Front Door with WAF',
            'api_management': 'API Management with WAF'
        }
        
        for template in templates:
            desc = descriptions.get(template, 'WAF enabled template')
            table.add_row(template, desc)
            
        console.print(table)
        
    except Exception as e:
        console.print(f"❌ Error: {str(e)}", style="red")
        raise click.Abort()


if __name__ == '__main__':
    main()


# azure-waf-deployer/azure_waf_deployer/__init__.py
"""
Azure WAF Deployer - A comprehensive package for deploying WAF-enabled infrastructure on Azure
"""

__version__ = "1.0.0"
__author__ = "Your Name"
__email__ = "your.email@example.com"

from .deployer import WAFDeployer
from .templates import TemplateManager
from .config import WAFConfig

__all__ = ["WAFDeployer", "TemplateManager", "WAFConfig"]


# azure-waf-deployer/azure_waf_deployer/deployer.py
"""
Main deployment class for Azure WAF infrastructure
"""

import json
import logging
from typing import Dict, List, Optional, Any
from pathlib import Path

from azure.identity import DefaultAzureCredential
from azure.mgmt.resource import ResourceManagementClient
from azure.mgmt.network import NetworkManagementClient
from azure.core.exceptions import AzureError

from .templates import TemplateManager
from .config import WAFConfig
from .exceptions import WAFDeploymentError

logger = logging.getLogger(__name__)


class WAFDeployer:
    """Azure WAF deployment manager"""
    
    def __init__(self, subscription_id: str, credential: Optional[Any] = None):
        """
        Initialize WAF Deployer
        
        Args:
            subscription_id: Azure subscription ID
            credential: Azure credential object (defaults to DefaultAzureCredential)
        """
        self.subscription_id = subscription_id
        self.credential = credential or DefaultAzureCredential()
        self.resource_client = ResourceManagementClient(self.credential, subscription_id)
        self.network_client = NetworkManagementClient(self.credential, subscription_id)
        self.template_manager = TemplateManager()
        
    def deploy_application_gateway_waf(
        self,
        resource_group: str,
        config: WAFConfig,
        template_type: str = "application_gateway"
    ) -> Dict[str, Any]:
        """
        Deploy Application Gateway with WAF
        
        Args:
            resource_group: Resource group name
            config: WAF configuration
            template_type: Template type to use
            
        Returns:
            Deployment result dictionary
        """
        try:
            logger.info(f"Starting Application Gateway WAF deployment in {resource_group}")
            
            # Get template and parameters
            template = self.template_manager.get_template(template_type)
            parameters = config.get_deployment_parameters()
            
            # Create resource group if it doesn't exist
            self._ensure_resource_group(resource_group, config.location)
            
            # Deploy ARM template
            deployment_name = f"waf-deployment-{config.name}"
            deployment_operation = self.resource_client.deployments.begin_create_or_update(
                resource_group_name=resource_group,
                deployment_name=deployment_name,
                parameters={
                    'properties': {
                        'template': template,
                        'parameters': parameters,
                        'mode': 'Incremental'
                    }
                }
            )
            
            # Wait for completion
            result = deployment_operation.result()
            logger.info(f"Deployment completed successfully: {deployment_name}")
            
            return {
                'deployment_name': deployment_name,
                'status': 'Success',
                'outputs': result.properties.outputs if result.properties.outputs else {},
                'resource_group': resource_group
            }
            
        except AzureError as e:
            logger.error(f"Azure deployment error: {str(e)}")
            raise WAFDeploymentError(f"Failed to deploy WAF: {str(e)}")
        except Exception as e:
            logger.error(f"Unexpected error during deployment: {str(e)}")
            raise WAFDeploymentError(f"Deployment failed: {str(e)}")
    
    def deploy_front_door_waf(
        self,
        resource_group: str,
        config: WAFConfig,
        template_type: str = "front_door"
    ) -> Dict[str, Any]:
        """
        Deploy Azure Front Door with WAF
        
        Args:
            resource_group: Resource group name
            config: WAF configuration
            template_type: Template type to use
            
        Returns:
            Deployment result dictionary
        """
        try:
            logger.info(f"Starting Front Door WAF deployment in {resource_group}")
            
            template = self.template_manager.get_template(template_type)
            parameters = config.get_deployment_parameters()
            
            self._ensure_resource_group(resource_group, config.location)
            
            deployment_name = f"frontdoor-waf-deployment-{config.name}"
            deployment_operation = self.resource_client.deployments.begin_create_or_update(
                resource_group_name=resource_group,
                deployment_name=deployment_name,
                parameters={
                    'properties': {
                        'template': template,
                        'parameters': parameters,
                        'mode': 'Incremental'
                    }
                }
            )
            
            result = deployment_operation.result()
            logger.info(f"Front Door deployment completed: {deployment_name}")
            
            return {
                'deployment_name': deployment_name,
                'status': 'Success',
                'outputs': result.properties.outputs if result.properties.outputs else {},
                'resource_group': resource_group
            }
            
        except AzureError as e:
            logger.error(f"Azure Front Door deployment error: {str(e)}")
            raise WAFDeploymentError(f"Failed to deploy Front Door WAF: {str(e)}")
    
    def validate_template(self, resource_group: str, config: WAFConfig, template_type: str) -> bool:
        """
        Validate ARM template before deployment
        
        Args:
            resource_group: Resource group name
            config: WAF configuration
            template_type: Template type to validate
            
        Returns:
            True if template is valid
        """
        try:
            template = self.template_manager.get_template(template_type)
            parameters = config.get_deployment_parameters()
            
            validation = self.resource_client.deployments.validate(
                resource_group_name=resource_group,
                deployment_name="validation-test",
                parameters={
                    'properties': {
                        'template': template,
                        'parameters': parameters,
                        'mode': 'Incremental'
                    }
                }
            )
            
            if validation.error:
                logger.error(f"Template validation failed: {validation.error}")
                return False
                
            logger.info("Template validation successful")
            return True
            
        except Exception as e:
            logger.error(f"Template validation error: {str(e)}")
            return False
    
    def _ensure_resource_group(self, resource_group: str, location: str):
        """Ensure resource group exists"""
        try:
            self.resource_client.resource_groups.get(resource_group)
            logger.info(f"Resource group {resource_group} already exists")
        except:
            logger.info(f"Creating resource group {resource_group}")
            self.resource_client.resource_groups.create_or_update(
                resource_group,
                {'location': location}
            )
    
    def get_deployment_status(self, resource_group: str, deployment_name: str) -> Dict[str, Any]:
        """Get deployment status"""
        try:
            deployment = self.resource_client.deployments.get(resource_group, deployment_name)
            return {
                'name': deployment.name,
                'provisioning_state': deployment.properties.provisioning_state,
                'timestamp': deployment.properties.timestamp,
                'outputs': deployment.properties.outputs
            }
        except Exception as e:
            logger.error(f"Error getting deployment status: {str(e)}")
            return {'error': str(e)}


# azure-waf-deployer/azure_waf_deployer/config.py
"""
Configuration management for WAF deployments
"""

import yaml
from typing import Dict, List, Optional, Any
from dataclasses import dataclass, field
from pathlib import Path

@dataclass
class CustomRule:
    """Custom WAF rule configuration"""
    name: str
    priority: int
    rule_type: str
    action: str
    match_conditions: List[Dict[str, Any]]
    enabled: bool = True

@dataclass
class ManagedRuleSet:
    """Managed rule set configuration"""
    rule_set_type: str
    rule_set_version: str
    exclusions: List[Dict[str, Any]] = field(default_factory=list)
    rule_group_overrides: List[Dict[str, Any]] = field(default_factory=list)

@dataclass
class WAFConfig:
    """WAF configuration class"""
    name: str
    location: str
    sku_tier: str = "WAF_v2"
    sku_name: str = "WAF_v2"
    capacity: int = 2
    waf_mode: str = "Prevention"  # Detection or Prevention
    
    # Managed rules
    managed_rule_sets: List[ManagedRuleSet] = field(default_factory=lambda: [
        ManagedRuleSet("Microsoft_DefaultRuleSet", "2.1"),
        ManagedRuleSet("Microsoft_BotManagerRuleSet", "1.0")
    ])
    
    # Custom rules
    custom_rules: List[CustomRule] = field(default_factory=list)
    
    # Network configuration
    vnet_name: str = ""
    subnet_name: str = ""
    public_ip_name: str = ""
    
    # Backend configuration
    backend_pools: List[Dict[str, Any]] = field(default_factory=list)
    
    # SSL configuration
    ssl_certificates: List[Dict[str, Any]] = field(default_factory=list)
    
    # Additional settings
    enable_http2: bool = True
    request_timeout: int = 20
    
    @classmethod
    def from_yaml(cls, file_path: str) -> 'WAFConfig':
        """Load configuration from YAML file"""
        with open(file_path, 'r') as f:
            data = yaml.safe_load(f)
        return cls(**data)
    
    def to_yaml(self, file_path: str):
        """Save configuration to YAML file"""
        with open(file_path, 'w') as f:
            yaml.dump(self.__dict__, f, default_flow_style=False)
    
    def get_deployment_parameters(self) -> Dict[str, Any]:
        """Convert config to ARM template parameters"""
        return {
            "applicationGatewayName": {"value": self.name},
            "location": {"value": self.location},
            "skuTier": {"value": self.sku_tier},
            "skuName": {"value": self.sku_name},
            "capacity": {"value": self.capacity},
            "wafMode": {"value": self.waf_mode},
            "managedRuleSets": {"value": [
                {
                    "ruleSetType": mrs.rule_set_type,
                    "ruleSetVersion": mrs.rule_set_version,
                    "exclusions": mrs.exclusions,
                    "ruleGroupOverrides": mrs.rule_group_overrides
                } for mrs in self.managed_rule_sets
            ]},
            "customRules": {"value": [
                {
                    "name": rule.name,
                    "priority": rule.priority,
                    "ruleType": rule.rule_type,
                    "action": rule.action,
                    "matchConditions": rule.match_conditions,
                    "enabled": rule.enabled
                } for rule in self.custom_rules
            ]},
            "vnetName": {"value": self.vnet_name},
            "subnetName": {"value": self.subnet_name},
            "publicIpName": {"value": self.public_ip_name},
            "backendPools": {"value": self.backend_pools},
            "sslCertificates": {"value": self.ssl_certificates},
            "enableHttp2": {"value": self.enable_http2},
            "requestTimeout": {"value": self.request_timeout}
        }


# azure-waf-deployer/azure_waf_deployer/exceptions.py
"""
Custom exceptions for WAF deployment
"""

class WAFDeploymentError(Exception):
    """Raised when WAF deployment fails"""
    pass

class TemplateError(Exception):
    """Raised when template operations fail"""
    pass

class ConfigurationError(Exception):
    """Raised when configuration is invalid"""
    pass


# azure-waf-deployer/setup.py
from setuptools import setup, find_packages
import os

# Read the contents of README file
this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="azure-waf-deployer",
    version="1.0.0",
    author="Your Name",
    author_email="your.email@example.com",
    description="Azure Web Application Firewall deployment templates and utilities",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yourusername/azure-waf-deployer",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    python_requires=">=3.8",
    install_requires=[
        "azure-identity>=1.12.0",
        "azure-mgmt-resource>=22.0.0",
        "azure-mgmt-network>=25.0.0",
        "azure-mgmt-web>=7.0.0",
        "azure-cli-core>=2.45.0",
        "jinja2>=3.1.2",
        "pyyaml>=6.0",
        "click>=8.0.0",
        "rich>=13.0.0",
    ],
    extras_require={
        "dev": [
            "pytest>=7.0.0",
            "black>=22.0.0",
            "flake8>=5.0.0",
            "mypy>=1.0.0",
        ],
    },
    include_package_data=True,
    package_data={
        "azure_waf_deployer": [
            "templates/*.json",
            "templates/*.bicep",
            "templates/*.yml",
            "configs/*.yaml",
        ],
    },
    entry_points={
        "console_scripts": [
            "azure-waf-deploy=azure_waf_deployer.cli:main",
        ],
    },
    project_urls={
        "Bug Reports": "https://github.com/yourusername/azure-waf-deployer/issues",
        "Source": "https://github.com/yourusername/azure-waf-deployer",
        "Documentation": "https://azure-waf-deployer.readthedocs.io/",
    },
)